@@ -47,8 +47,8 @@ from ._version import __frontend_version__
 
 
 def register_scale(key=None):
-
-    """Returns a decorator to register a scale type in the scale type registry.
+    """Returns a decorator to register a scale type in the scale type
+    registry.
 
     If no key is provided, the class name is used as a key. A key is
     provided for each core bqplot scale type so that the frontend can use
@@ -353,7 +353,7 @@ class LinearScale(Scale):
         LinearScale should take precedence.
     stabilized: bool (default: False)
         if set to False, the domain of the scale is tied to the data range
-        if set to True, the domain of the scale is udpated only when
+        if set to True, the domain of the scale is updated only when
         the data range is beyond certain thresholds, given by the attributes
         mid_range and min_range.
     mid_range: float (default: 0.8)
@@ -455,7 +455,7 @@ class OrdinalScale(Scale):
         the associated data type / domain type
     """
     rtype = 'Number'
-    dtype = np.str
+    dtype = np.str_
     domain = List().tag(sync=True)
 
     _view_name = Unicode('OrdinalScale').tag(sync=True)
@@ -483,6 +483,8 @@ class ColorScale(Scale):
         if not None, mid is the value corresponding to the mid color.
     scheme: string (default: 'RdYlGn')
         Colorbrewer color scheme of the color scale.
+    extrapolation: {'constant', 'linear'} (default: 'constant')
+        How to extrapolate values outside the [min, max] domain.
     rtype: string (class-level attribute)
         The range type of a color scale is 'Color'. This should not be modifed.
     dtype: type (class-level attribute)
@@ -491,11 +493,13 @@ class ColorScale(Scale):
     rtype = 'Color'
     dtype = np.number
     scale_type = Enum(['linear'], default_value='linear').tag(sync=True)
-    colors = List(trait=Color(default_value=None, allow_none=True)).tag(sync=True)
+    colors = List(trait=Color(default_value=None, allow_none=True))\
+        .tag(sync=True)
     min = Float(None, allow_none=True).tag(sync=True)
     max = Float(None, allow_none=True).tag(sync=True)
     mid = Float(None, allow_none=True).tag(sync=True)
     scheme = Unicode('RdYlGn').tag(sync=True)
+    extrapolation = Enum(['constant', 'linear'], default_value='constant').tag(sync=True)
 
     _view_name = Unicode('ColorScale').tag(sync=True)
     _model_name = Unicode('ColorScaleModel').tag(sync=True)
@@ -522,8 +526,12 @@ class DateColorScale(ColorScale):
     dtype: type (class-level attribute)
         the associated data type / domain type
     """
-    rtype = 'Color'
     dtype = np.datetime64
+    domain_class = Type(Date)
+
+    min = Date(default_value=None, allow_none=True).tag(sync=True)
+    mid = Date(default_value=None, allow_none=True).tag(sync=True)
+    max = Date(default_value=None, allow_none=True).tag(sync=True)
 
     _view_name = Unicode('DateColorScale').tag(sync=True)
     _model_name = Unicode('DateColorScaleModel').tag(sync=True)
@@ -547,7 +555,7 @@ class OrdinalColorScale(ColorScale):
         the associated data type / domain type
     """
     rtype = 'Color'
-    dtype = np.str
+    dtype = np.str_
     domain = List().tag(sync=True)
 
     _view_name = Unicode('OrdinalColorScale').tag(sync=True)
